;**********************************************************************
;                                                                     *
;    Filename:	    main628.asm                                       *
;    Date:                                                            *
;    File Version:                                                    *
;                                                                     *
;    Author:        el@jap.hu                                         *
;                   http://jap.hu/                                    *
;**********************************************************************
; HISTORY
;
; 000 - 20010620 initial version
; 001 - 20100311 adapted to pbus.069
; 002 - 20100402 correct devid setting, MCLRE fuse
; 003 - 20100402 demonstration pbus_handler with digital I/O handling
;**********************************************************************
;
;    Notes:
;**********************************************************************

	list      p=16F628             ; list directive to define processor
	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _HS_OSC & _LVP_OFF & _MCLRE_OFF

#include <p16f628.inc>
#include "pbus628.inc"
#include "lcdlib.inc"

freemem UDATA

;***** VARIABLES

w_save		RES 1
s_save		RES 1
tmp1		equ 0x70
tmp2		equ 0x71

vectors		CODE 0

		nop
  		goto    main              ; go to beginning of program
		nop
		nop
		goto int_handler

		CODE 5
messages	movwf PCL
msg_1		dt "Pbus-069 slave", 0

int_handler	movwf w_save
		swapf STATUS, W
		movwf s_save
		call pbus_int_handler

		swapf s_save, W
		movwf STATUS
		swapf w_save
		swapf w_save, W
		retfie

main		clrf PORTA
		clrf PORTB
		movlw 0
		tris PORTA
		movlw 0
		tris PORTB
		movlw .7
		movwf CMCON ; all inputs are digital

		movlw 0x50 ; device id in higher four bits!
		call pbus_init
		call lcd_init

		movlw msg_1
		call lcd_strout

loop		movlw 0xc0
		call lcd_cmdout

		; DISPLAY: PIE, PIR, T1CON, RCSTA, TXSTA

		bcf INTCON, GIE
		BANKSEL PIE1
		movf PIE1, W
		movwf tmp1
		movf TXSTA, W
		movwf tmp2
		BANKSEL PIR1
		bsf INTCON, GIE
		movf tmp1, W; PIE1
		call lcd_hexout
		movf PIR1, W
		call lcd_hexout
		movf T1CON, W
		call lcd_hexout
		movf RCSTA, W
		call lcd_hexout
		movf tmp2, W; TXSTA
		call lcd_hexout

		; DISPLAY: FSR, bytecnt

		movf FSR, W
		call lcd_hexout
		EXTERN bytecnt
		movf bytecnt, W
		call lcd_hexout

		movlw 0x8a
		call lcd_cmdout

		; DISPLAY: counters
		movf rxe1cnt, W
		call lcd_hexout
		movf rxe2cnt, W
		call lcd_hexout
		movf rxe3cnt, W
		call lcd_hexout

		goto loop

pbus_handler	;
		; This is where the custom pbus commands get handled
		; The handler is called with command in the W register
		; and returns W = 0 if there is a response in the tx buffer
		; or W != 0 if there is no response (invalid command)
		;
		; Important: the handler is called from an interrupt!
		;
		; receive buffer: rxbuf
		; transmit buffer: txbuf
		;

		xorlw 0x54 ; set/get port value
		bnz not_54

		; handle command 0x54
		; 54 - get PORTA state
		; 54 ## - set PORTA state

		movlw ROK
		movwf txbuf+1 ; response code is OK
		movlw 0x0f
		andwf rxbuf, W
		bz cmd54_get ; request length == 0 ? (get or set)

cmd54_set	movf rxbuf+2, W ; request_length > 0, set
		movwf PORTA
		clrf txbuf ; response length is zero
		retlw 0 ; done
cmd54_get	movf PORTA, W
		movwf txbuf+2 ; response first byte
		movlw .1
		movwf txbuf ; response length
		retlw 0 ; done

not_54		xorlw 0x54
		xorlw 0x55
		bnz not_55

		; handle command 0x55
		; 55 - get TRISA state
		; 55 ## - set TRISA state

		movlw ROK
		movwf txbuf+1 ; response code is OK
		movlw 0x0f
		andwf rxbuf, W
		bz cmd55_get ; request length == 0 ? (get or set)

cmd55_set	movf rxbuf+2, W ; request_length > 0, set
		BANKSEL TRISA
		movwf TRISA
		BANKSEL PORTA
		clrf txbuf ; response length is zero
		retlw 0 ; done
cmd55_get	
		BANKSEL TRISA
		movf TRISA, W
		BANKSEL PORTA
		movwf txbuf+2 ; response first byte
		movlw .1
		movwf txbuf ; response length
		retlw 0 ; done

not_55		retlw 0xff ; no valid cmd, no response
		end
